
import { initAir } from './air';
import { playAudio, showBigStar } from './util';
cc.Class({
	extends: cc.Component,
	properties: {
	},

	callNetworkApiPost(uri, data, callBack) {
		this.getBaseUrl((baseUrl) => {
			const xhr = new XMLHttpRequest();
			const url = `${baseUrl}${uri}`;
			xhr.open("POST", url, true);
			xhr.setRequestHeader('content-type', 'application/json');
			xhr.onreadystatechange = () => {
				if (xhr.readyState == 4) {
					callBack(JSON.parse(xhr.responseText));
				}
			}
			xhr.send(JSON.stringify(data));
		});
	},

	callNetworkApiGet(uri, data, callBack) {
		this.getBaseUrl((baseUrl) => {
			let queryStr = '?';
			const params = [];
			for (const key in data) {
				if (Object.hasOwnProperty.call(data, key)) {
					params.push(`${key}=${data[key]}`);
				}
			}
			queryStr += params.join("&");
			const xhr = new XMLHttpRequest();
			xhr.onreadystatechange = () => {
				if (xhr.readyState == 4 && (xhr.status >= 200 && xhr.status < 400)) {
					callBack(JSON.parse(xhr.responseText));
				}
			};
			const url = `${baseUrl}${uri}${queryStr}`;
			console.warn('url = ' + url);
			xhr.open('GET', url, true);
			xhr.send();
		});
	},

	async getBaseUrl(callback) {
		const engineInfo = await this.getEngineInfo();
		const { isDev, uuid } = JSON.parse(engineInfo);
		this.isDev = isDev;
		this.engineCode = uuid;
		if (isDev == 1) {
			callback('http://staging-openapi.iteachabc.com');
		} else {
			callback('http://openapi.iteachabc.com');
		}
	},

	getEngineInfo() {
		if (window.air.engineInfo) {
			return window.air.engineInfo;
		} else {
			return new Promise((resolve, reject) => {
				window.courseware.getEngineInfo(() => {
					resolve(window.air.engineInfo);
				});
			});
		}
	},
	asyncCallNetworkApiGet(apiName, data) {
		return new Promise((resolve, reject) => {
			this.callNetworkApiGet(apiName, data, (res => {
				resolve(res);
			}));
		});
	},
	asyncCallNetworkApiPost(uri, data) {
		return new Promise((resolve, reject) => {
			this.callNetworkApiPost(uri, data, (res) => {
				resolve(res);
			});
		});
	},

	onMiddleLayerLoadingFinish() {
		if (cc.sys.isNative && cc.sys.os == cc.sys.OS_IOS) {
			jsb.reflection.callStaticMethod(
				"RootViewController",
				"getMiddleLayerInfoFinish:",
				""
			);
		}
		if (cc.sys.isNative && cc.sys.os == cc.sys.OS_ANDROID) {
			jsb.reflection.callStaticMethod(
				"com/iplayabc/cocos/AppActivity",
				"getMiddleLayerInfoFinish",
				"(Ljava/lang/String;)Ljava/lang/String;",
				""
			);
		}
	},
	 

	// 需要预加载的数量
	preloadCount: null,
	// 统计已加载的数量
	loadedCount: null,

	isDev: null,
	engineCode: null, // 机器唯一编码

	dataStore: null, // 内存数据仓库
	currentSceneName: null, // 当前模板名称

	// 统一释放异步资源的方法
	onLoad() {
		this.preloadCount = 0;
		this.loadedCount = 0;

		this.dataStore = {};
		initAir(this);
		this.reWriteAir();
	},

	start() {
		this.node.zIndex = 9999;
		cc.game.addPersistRootNode(this.node);


		this.initListener();
		this.showWaitingLetters();
		this.onMiddleLayerLoadingFinish();

		this.loadOnlineBundle("OP_03_2");
	},

	initListener() {
  
	},

	reWriteAir() {
    window.courseware.getData = async (callback) => {
      const data = this.dataStore[this.currentSceneName];
      callback && callback(data);
    }
    if (!window.air) {
      window.air = {};
    }
    window.air.hideAirClassLoading = () => {
      window.air.onCourseInScreen && window.air.onCourseInScreen(() => {
        console.log("***成功调用onCourseInScreen***");
        window.air.onCourseInScreen = null;
      });
      this.hideWaitingLetters();
    }
  },
  
	async loadOnlineBundle(bundleName) {
		cc.audioEngine.stopAll();
		this.currentSceneName = bundleName;

		this.showWaitingLetters();
		const data = await this.asyncCallNetworkApiGet(`/api/template/v1/${bundleName}`, {});
		let configData = null;
		if (cc.sys.os == cc.sys.OS_IOS) {
			configData = data.data.conf.ios;
		} else if (cc.sys.os == cc.sys.OS_ANDROID) {
			configData = data.data.conf.android;
		} else {
			configData = data.data.conf.web_desktop;
		}
		const sceneName = configData.sceneName;
		const version = configData.version;
		const bondleUrl = configData.bondleUrl;
		this.loadBundle(sceneName, version, bondleUrl);
	},
	loadBundle(sceneName, version, bondleUrl) {
		cc.assetManager.loadBundle(bondleUrl, { version: version }, async (err, bundle) => {
			if (err) {
				console.error(err);
				return;
			}
			bundle.loadScene(sceneName, null, null, (err, scene) => {
				if (err) {
					console.error(err);
				}
				cc.director.runScene(scene, null, () => {
					const Canvas = cc.find('Canvas');
					const middleLayer = cc.find('middleLayer');
					Canvas.getComponent(cc.Widget).updateAlignment();
					middleLayer.scale = Canvas.width / middleLayer.width;
				});
			});
		});
	},

	showWaitingLetters() {
		if (this.isShowWaiting) {
			return;
		}
		this.isShowWaiting = true;

		const colorList = this.getRainbowColorList();
		const layout = cc.find('middleLayer/layout');
		layout.removeAllChildren();
		layout.active = true;
		const str = 'Now Loading...';
		str.split('').forEach((word, idx) => {
			const node = new cc.Node();
			const label = node.addComponent(cc.Label);
			label.string = word;
			node.parent = layout;
			node.color = colorList[idx];
			cc.tween(node)
				.delay(idx / 4)
				.by(0.3, { y: 50 }, { easing: 'sineOut' })
				.by(0.3, { y: -50 }, { easing: 'sineIn' })
				.delay((str.length - idx) / 4)
				.union()
				.repeatForever()
				.start();
		});
		const totalWidth = layout.children.reduce((width, node, idx) => {
			return width + node.width;
		}, 0);
		layout.width = totalWidth;
		
	},

	hideWaitingLetters() {
		const layout = cc.find('middleLayer/layout');
		layout.active = false;
		this.isShowWaiting = false;
	},

	getRainbowColorList() {
		return [
			cc.color(255, 255, 0),
			cc.color(128, 255, 0),
			cc.color(0, 255, 0),
			cc.color(0, 0, 0),

			cc.color(0, 255, 128),
			cc.color(0, 255, 255),

			cc.color(0, 128, 255),
			cc.color(0, 0, 255),
			cc.color(128, 0, 255),
			cc.color(255, 0, 255),
			cc.color(255, 0, 128),

			cc.color(255, 0, 0),
			cc.color(255, 128, 0),
			cc.color(255, 255, 0),
		];
	},

	log(str) {
		console.log(str);
	}

});
